@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import ctypes
+from ctypes import Array, c_uint32
 
 from einspect.structs.deco import struct
 from einspect.structs.py_object import PyVarObject
@@ -13,25 +14,42 @@ class PyLongObject(PyVarObject):
 
     https://github.com/python/cpython/blob/3.11/Include/cpython/longintrepr.h#L79-L82
     """
+
     _ob_digit_0: ctypes.c_uint32 * 0
 
+    @classmethod
+    def _format_fields_(cls) -> dict[str, str]:
+        return super()._format_fields_() | {
+            "ob_digit": "Array[c_uint32]"
+        }
+
     @property
     def mem_size(self) -> int:
         """Return the size of the PyObject in memory."""
         # Need to add size(uint32) * ob_size to our base size
         base = super().mem_size
-        return base + ctypes.sizeof(ctypes.c_uint32) * self.ob_size
+        # use size 1 if ob_size is 0 due to allocation
+        size = min(1, abs(self.ob_size))
+        return base + ctypes.sizeof(c_uint32) * size
 
     @property
-    def ob_digit(self):
+    def ob_digit(self) -> Array[c_uint32]:
         # Note PyLongObject uses the sign bit of ob_size to indicate its own sign
         # ob_size < 0 means the number is negative
         # ob_size > 0 means the number is positive
         # ob_size == 0 means the number is zero
         # The true size of the ob_digit array is abs(ob_size)
         items_addr = ctypes.addressof(self._ob_digit_0)
-        size = abs(int(self.ob_size))  # type: ignore
-        return (ctypes.c_uint32 * size).from_address(items_addr)
+        size = max(abs(int(self.ob_size)), 1)  # type: ignore
+        return (c_uint32 * size).from_address(items_addr)
+
+    @ob_digit.setter
+    def ob_digit(self, value: Array[c_uint32]) -> None:
+        ctypes.memmove(
+            ctypes.addressof(self._ob_digit_0),
+            ctypes.addressof(value),
+            ctypes.sizeof(value),
+        )
 
     @property
     def value(self) -> int:
