@@ -1,20 +1,27 @@
 from __future__ import annotations
 
-from ctypes import Array
+from ctypes import Array, sizeof, c_uint32
 from typing import Iterable, TypeVar
 
-from einspect.api import Py_ssize_t
 from einspect.structs import PyLongObject
 from einspect.views.view_base import VarView
 
+__all__ = ("IntView",)
+
 _T = TypeVar("_T")
 
 
-class IntView(VarView[_T]):
+class IntView(VarView[int, None, None]):
     _pyobject: PyLongObject
 
     @property
-    def digits(self) -> Array[Py_ssize_t]:
+    def mem_size(self) -> int:
+        size = min(1, self.size)
+        arr_size = size * sizeof(c_uint32)
+        return sizeof(PyLongObject) + arr_size
+
+    @property
+    def digits(self) -> Array[c_uint32]:
         return self._pyobject.ob_digit
 
     @digits.setter
@@ -35,7 +42,7 @@ class IntView(VarView[_T]):
 
         # The new value's ob_size must be equal or less than the current
         new_size = abs(new_val.ob_size)
-        cur_size = abs(self._pyobject.ob_size)
+        cur_size = max(abs(self._pyobject.ob_size), 1)
         if new_size > cur_size:
             raise ValueError(f"New value {obj!r} too large")
 
